package wood.keith.opentools.ibxdesigner;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import com.borland.internetbeans.IxComponent;
import com.borland.internetbeans.PageProducer;
import com.borland.jbuilder.cmt.CmtComponent;
import com.borland.jbuilder.cmt.CmtComponentSource;
import com.borland.jbuilder.cmt.CmtModel;
import com.borland.jbuilder.cmt.CmtModelNode;
import com.borland.jbuilder.cmt.CmtSubcomponent;

/**
 * The model that contains all the InternetBeans components.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  10 September 2001
 */
public class InternetBeansModel implements CmtModel {

  private InternetBeansDesigner _designer;
  private CmtComponentSource _source;
  private DefaultTreeModel _tree;
  private String _name;

  /**
   * Initialise the model that manages the subtree for this designer.
   *
   * @param  designer  the designer instance that created this model
   * @param  source    the source reference that corresponds to this model
   * @param  tree      the tree of nodes managed by this model
   * @param  name      the name of the model
   */
  public InternetBeansModel(InternetBeansDesigner designer,
      CmtComponentSource source, DefaultTreeModel tree, String name) {
    _designer = designer;
    _source = source;
    _tree = tree;
    _name = name;
  }

  public boolean isMultiInstance() { return false; }

  /**
   * Does this sub-tree claim and manage the subcomponent.
   * By claiming a subcomponent, you prevent it from appearing
   * under another sub-tree.
   *
   * @param  subcomponent  the embedded component to be checked
   * @return  true if this model claims this component, false otherwise
   */
  public boolean isSubcomponentOwned(CmtSubcomponent subcomponent) {
    Object object = subcomponent.getLiveInstance();
    return (object instanceof PageProducer || object instanceof IxComponent);
  }

  public DefaultTreeModel getGraph() { return _tree; }

  public CmtComponent getComponent() { return _source; }

  public CmtModelNode[] getChildren(CmtModelNode parent) {
    CmtModelNode[] children = new CmtModelNode[parent.getChildCount()];
    for (int index = 0; index < parent.getChildCount(); index++) {
      children[index] = (CmtModelNode)parent.getChildAt(index);
    }
    return children;
  }

  public CmtModelNode getRoot() { return (CmtModelNode)_tree.getRoot(); }

  public CmtModelNode getParent(CmtModelNode child) {
    return (CmtModelNode)child.getParent();
  }

  /**
   * Add a new node to this subtree.
   *
   * @param  parent              the node to add the new node to
   * @param  componentClassName  the new class to instantiate
   * @param  aheadOf             the node to add the new node before
   * @param  subcomponent        a previously constructed wrapper for the new node,
   *                             or null if not yet handled
   * @return  the new node
   */
  public CmtModelNode add(CmtModelNode parent, String componentClassName,
      CmtModelNode aheadOf, CmtSubcomponent subcomponent) {
    CmtSubcomponent subcomp = _source.addSubcomponent(
        componentClassName, null, CmtSubcomponent.CLASS_SCOPE);
    InternetBeansModelNode node =
        new InternetBeansModelNode(_designer, this, subcomp);
    _tree.insertNodeInto(node, (DefaultMutableTreeNode)parent,
        (aheadOf == null ? 0 : parent.getIndex(aheadOf)));
    _source.commit(false);
    return node;
  }

  /**
   * Move a node to a new location.
   *
   * @param  node       the node to relocate
   * @param  newParent  the node to add the new node to
   * @param  aheadOf    the node to add the new node before
   * @return  the moved node
   */
  public CmtModelNode move(CmtModelNode node, CmtModelNode newParent,
      CmtModelNode aheadOf) {
    _tree.insertNodeInto((DefaultMutableTreeNode)node,
        (DefaultMutableTreeNode)newParent,
        (aheadOf == null ? -1 : newParent.getIndex(aheadOf)));
    return node;
  }

  /**
   * Delete a node from the tree.
   *
   * @param  node  the node to delete
   */
  public void remove(CmtModelNode node) {
    _source.removeSubcomponent(node.getSubcomponent());
    _tree.removeNodeFromParent((DefaultMutableTreeNode)node);
    _source.commit(false);
  }

  public String getName() { return _name; }

  public void close(){}
}
