package wood.keith.opentools.ibxdesigner;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Enumeration;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import com.borland.internetbeans.IxComponent;
import com.borland.internetbeans.PageProducer;
import com.borland.jbuilder.cmt.CmtModel;
import com.borland.jbuilder.cmt.CmtModelNode;
import com.borland.jbuilder.cmt.CmtSubcomponent;

/**
 * The tree node that encapsulates the InternetBeans component.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  10 September 2001
 */
public class InternetBeansModelNode extends DefaultMutableTreeNode
    implements CmtModelNode, PropertyChangeListener {

  private InternetBeansDesigner _designer;
  private CmtModel _model;
  private CmtSubcomponent _subcomponent;
  private String _name;

  /**
   * Initialise the node for this subcomponent.
   *
   * @param  designer      the designer instance that created this node
   * @param  model         the model that manages this subtree
   * @param  subcomponent  the component encapsulated by this node
   * @param  name          the name of the node
   */
  public InternetBeansModelNode(InternetBeansDesigner designer, CmtModel model,
      CmtSubcomponent subcomponent, String name) {
    _designer = designer;
    _model = model;
    _subcomponent = subcomponent;
    _name = name;
    if (subcomponent != null) {
      subcomponent.addPropertyChangeListener(this);
    }
  }

  /**
   * Initialise the node for this subcomponent.
   *
   * @param  designer      the designer instance that created this node
   * @param  model         the model that manages this subtree
   * @param  subcomponent  the component encapsulated by this node
   */
  public InternetBeansModelNode(InternetBeansDesigner designer, CmtModel model,
      CmtSubcomponent subcomponent) {
    this(designer, model, subcomponent,
        (subcomponent == null ? "null" : subcomponent.getName()));
  }

  public String getName() { return _name; }

  public void setName(String value) { _name = value; }

  public CmtModel getCmtModel() { return _model; }

  public CmtSubcomponent getSubcomponent() { return _subcomponent; }

  public String getTag() { return null; }

  public boolean isDesignable() { return true; }

  /**
   * Update the node, including its position, based on the component's
   * property values.
   *
   * @param  evt  the triggering event
   */
  public void propertyChange(PropertyChangeEvent evt) {
    if (evt.getPropertyName().equals("<Aname>")) {
      // Change the displayed name
      setName((String)evt.getNewValue());
      getCmtModel().getGraph().nodeChanged(this);
    }
    else if (evt.getPropertyName().equals("pageProducer")) {
      // Reposition the component under its page producer
      DefaultTreeModel tree = getCmtModel().getGraph();
      tree.removeNodeFromParent(this);
      PageProducer producer =
          ((IxComponent)getSubcomponent().getLiveInstance()).getPageProducer();
      InternetBeansModelNode parent =
          (InternetBeansModelNode)getCmtModel().getRoot();
      Enumeration children = parent.children();
      while (children.hasMoreElements()) {
        InternetBeansModelNode child =
            (InternetBeansModelNode)children.nextElement();
        if (child.getSubcomponent().getLiveInstance() == producer) {
          parent = child;
          break;
        }
      }
      tree.insertNodeInto(this, parent, 0);
      // And update the designer viewer
      _designer.setPageProducer(producer,
          (producer == null ? getName() : parent.getName()));
      _designer.setSelection(getSubcomponent().getLiveInstance());
    }
    else if (evt.getPropertyName().equals("controlName") ||
        evt.getPropertyName().equals("elementId")) {
      // Highlight selected component
      _designer.setSelection(getSubcomponent().getLiveInstance());
    }
  }

  public String toString() { return getName(); }
}
