package wood.keith.opentools.ibxdesigner;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.io.IOException;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

//import com.borland.jbuilder.editor.HTMLEditorKit;  // JB9-
import com.borland.internetbeans.AbstractIxControl;
import com.borland.internetbeans.IxComponent;
import com.borland.internetbeans.PageProducer;
import com.borland.primetime.editor.EditorManager;
import com.borland.primetime.editor.EditorPane;
import com.borland.primetime.editor.TextEditorKit;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.html.HtmlEditorKit;  // JB10+
import com.borland.primetime.vfs.Buffer;
import com.borland.primetime.vfs.BufferListener;
import com.borland.primetime.vfs.BufferUpdater;
import com.borland.primetime.vfs.ReadOnlyException;
import com.borland.primetime.vfs.Url;

/**
 * The UI for the InternetBeans designer.
 * It displays the contents of the HTML file associated with a PageProducer
 * and highlights the position of IxComponents within that file.
 *
 * @author   Keith Wood (kbwood@iprimus.com.au)
 * @version  1.0  10 September 2001
 */
public class InternetBeansViewer extends JPanel
    implements BufferListener, BufferUpdater, DocumentListener {

  private BorderLayout _borderLayout = new BorderLayout();
  private JLabel _nameLabel = new JLabel();
  private JScrollPane _viewScroll = new JScrollPane();
  private EditorPane _viewPane = new EditorPane();
  /* Specialised editor kit for display */
//  private TextEditorKit _htmlEditorKit =  // JB9-
//      EditorManager.getEditorKit(HTMLEditorKit.class);
  private TextEditorKit _htmlEditorKit =  // JB10+
      EditorManager.getEditorKit(HtmlEditorKit.class);

  private PageProducer _producer = null;
  private Buffer _buffer = null;

  public InternetBeansViewer() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * JBuilder UI initialisation.
   *
   * @throws  Exception
   */
  private void jbInit() throws Exception {
    this.setLayout(_borderLayout);
    _nameLabel.setText("Component");
    _viewPane.setFont(EditorManager.getFont());
    _viewPane.setEditorKit(_htmlEditorKit);
    _viewPane.setText("Internet Beans Viewer");
    _viewPane.getDocument().addDocumentListener(this);
    _viewScroll.setPreferredSize(new Dimension(200, 200));
    this.add(_nameLabel,  BorderLayout.NORTH);
    this.add(_viewScroll,  BorderLayout.CENTER);
    _viewScroll.getViewport().add(_viewPane, null);
  }

  /**
   * Display the HTML content of the PageProducer,
   * or an appropriate message if it is not available.
   * Also tie into the VFS for changes to the HTML file buffer.
   *
   * @param  producer  the PageProducer to get the content from
   * @param  name      the component name to display at the top
   */
  public void setPageProducer(PageProducer producer, String name) {
    if (_buffer != null) {
      _buffer.removeBufferListener(this);
      _buffer = null;
    }
    _viewPane.getDocument().removeDocumentListener(this);
    _producer = producer;
    _nameLabel.setText("    " + name);
    _viewPane.setEditable(false);
    if (producer == null) {
      _viewPane.setText("Please set the page producer for this component");
      _viewPane.getDocument().addDocumentListener(this);
      return;
    }
    try {
      String filePath = null;
      try {
        filePath = producer.getHtmlPath();
      }
      catch (NullPointerException npe) {
        filePath = null;
      }
      if (filePath == null || filePath.length() == 0) {
        _viewPane.setText("Please set the HTML file name for this component");
      }
      else {
        _buffer = Browser.getActiveBrowser().getActiveProject().
            getNode(new Url("file", filePath)).getBuffer();
        _buffer.addBufferListener(this);
        _viewPane.setText(new String(_buffer.getContent()));
        _viewPane.setEditable(true);
      }
    }
    catch (IOException ioe) {
      ioe.printStackTrace();
      _viewPane.setText("Error during HTML load (" + ioe.getClass().getName() +
          ")\n" + ioe.getMessage());
    }
    _viewPane.getDocument().addDocumentListener(this);
  }

  /**
   * Mark the position of a component on the page.
   *
   * @param  component  the component to be highlighted
   */
  public void setSelection(IxComponent component) {
    Point loc = new Point(0, 0);
    if (component != null && component.getPageProducer() == _producer) {
      boolean useName = (component instanceof AbstractIxControl);
      String id = (useName ? ((AbstractIxControl)component).getControlName() :
          component.getElementId());
      String text = _viewPane.getText();
      String attrName = (useName ? "name" : "id");
      if (!findText(text, attrName + "=\"" + id + "\"", loc)) {
        if (!findText(text, attrName + " = \"" + id + "\"", loc)) {
          if (!findText(text, attrName + "='" + id + "'", loc)) {
            if (!findText(text, attrName + " = '" + id + "'", loc)) {
              findText(text, attrName + "=" + id, loc);
            }
          }
        }
      }
    }
    _viewPane.setSelectionStart((int)loc.getX());
    _viewPane.setSelectionEnd((int)loc.getY());
    _viewPane.repaint();
  }

  /**
   * Find the supplied text and return its position.
   *
   * @param  text     the text being searched
   * @param  subtext  the text to search for
   * @param  point    the location of the subtext (updated for output)
   * @return  true if the subtext was found, false otherwise
   */
  private boolean findText(String text, String subtext, Point point) {
    int start = text.indexOf(subtext);
    if (start == -1) {
      point.setLocation(0, 0);
      return false;
    }
    else {
      point.setLocation(start, start + subtext.length());
      return true;
    }
  }

  public PageProducer getPageProducer() { return _producer; }

  /**
   * If the buffer is changed by someone other than us, then reload the text.
   *
   * @param  buffer   the buffer being changed
   * @param  updater  the lazy updater for this buffer
   */
  public void bufferChanged(Buffer buffer, BufferUpdater updater) {
    if (updater != this) {
      _viewPane.getDocument().removeDocumentListener(this);
      _viewPane.setText(new String(buffer.getContent()));
      _viewPane.getDocument().addDocumentListener(this);
    }
  }

  public void bufferLoaded(Buffer buffer) {
    // Do nothing
  }

  public void bufferSaving(Buffer buffer) {
    // Do nothing
  }

  public void bufferStateChanged(Buffer buffer, int oldState, int newState) {
    // Do nothing
  }

  /**
   * Supply any changes to the file buffer as needed.
   *
   * @param  buffer  the buffer being updated
   * @return  the new content
   */
  public byte[] getBufferContent(Buffer buffer) {
    return _viewPane.getText().getBytes();
  }

  public void insertUpdate(DocumentEvent e) {
    doModified();
  }

  public void removeUpdate(DocumentEvent e) {
    doModified();
  }

  public void changedUpdate(DocumentEvent e) {
    doModified();
  }

  /**
   * Notify interested parties of changes to the file buffer.
   */
  private void doModified() {
    if (_buffer != null) {
      try {
        _buffer.setContent(this);
      }
      catch (ReadOnlyException ex) {
        // Ignore
      }
    }
  }
}
